import {
  Http,
  Metadata,
  I18n,
} from '@zeedhi/core';
import {
  Form,
  SelectMultiple,
  Select,
  LoadingService,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';
import BudgetParameters from '@/controllers/BudgetParameters';

export class BudgetPreparationController {
  /**
  * Searches for the parameters registered with the database to define the type of budget.
  * Fills the budget type field with the corresponding parameter and manipulates the dependent fields.
  *
  * @param {object} component - component that triggers the function.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public async mountedTypeBudget({ component }) {
    const utilities = new Utilities();
    await utilities.mountedParameters();
    utilities.budgetPeriod({ component });
  }
  
  /**
  * Assembles the necessary parameters for sending to the backend , title and message of the modal.
  * Opens the budget preparation confirmation modal.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public confirmPrepareBudget() {
    const utilities = new Utilities();
    const formPrepareBudget = Metadata.getInstance<Form>('budgetPreparationForm');
    const idPeriodoOrca = BudgetParameters.getParameters.IDPERIODOORCA;
    const nrItemOperacional = formPrepareBudget.value.NRITEMOPERACIONAL;
    let valuePeriod;
    if (idPeriodoOrca === 'ANUAL') {
      valuePeriod = 1;
    } else {
      valuePeriod = formPrepareBudget.value.NRPERIODOORCA.NRPERIODOORCA;
    }
    const stringTpOrca = this.typeBudgetString(nrItemOperacional);
    if (formPrepareBudget.validate()) {
      const dataForm = {
        NRANOORCAGOP: formPrepareBudget.value.NRANOORCAGOP,
        IDPERIODOORCA: idPeriodoOrca,
        NRPERIODOORCA: formPrepareBudget.value.NRPERIODOORCA,
        CDOPERADOR: formPrepareBudget.value.CDOPERADOR ? formPrepareBudget.value.CDOPERADOR : 'T',
        CDSUPERVISOR: formPrepareBudget.value.CDSUPERVISOR ? formPrepareBudget.value.CDSUPERVISOR : 'T',
        CDREGIAO: formPrepareBudget.value.CDREGIAO ? formPrepareBudget.value.CDREGIAO : 'T',
        CDGRUPFILI: formPrepareBudget.value.CDGRUPFILI ? formPrepareBudget.value.CDGRUPFILI : 'T',
        CDSEGMER: formPrepareBudget.value.CDSEGMER ? formPrepareBudget.value.CDSEGMER : 'T',
        CDFILIAL: formPrepareBudget.value.CDFILIAL ? formPrepareBudget.value.CDFILIAL : 'T',
        NRITEMOPERACIONAL: nrItemOperacional,
        VALUE_PERIOD: valuePeriod,
        STRINGTPORCA: stringTpOrca,
        TITLE: '',
        MESSAGE: I18n.translate('gop::EXIST_BUDGET'),
      };
      utilities.showDialog(this.prepareBudget, dataForm, 'YNA');
    }
  }
  
  /**
  * Includes the option clicked by the user in the row.
  * Makes the route call to the backend in order to prepare the budget.
  * In cases where one or more units could not be prepared,call the function to assemble the Log to be displayed to the user.
  *
  * @param {object} dataForm - parameters to be sent to the backend.
  * @param {string} type - Button that the user clicked whether Yes or No.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public prepareBudget(dataForm, type) {
    const utilities = new Utilities();
    dataForm.IDEXCLUIR = type;
    LoadingService.show();
    Http.post('/prepareBudget', {
      requestType: 'Row',
      row: dataForm,
    }).then((response) => {
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      const units = response.data.dataset.units;
      const list = response.data.dataset.errorList;
      if (!utilities.isEmptyOrBlank(list)) {
        let log = '';
        let countUnit = 0;
        list.forEach((item) => {
          const index = log.indexOf(item.CDFILIAL);
          if (index === -1) {
            const filial = `${item.CDFILIAL} - ${item.NMFILIAL}`;
            log = `${log}<b></br>${filial}</b></br>`;
            const motivo = I18n.translate(`gop::${item.DSMOTIVO}`);
            log = `${log}${motivo}</br>`;
            countUnit += 1;
          } else {
            const motivo = I18n.translate(`gop::${item.DSMOTIVO}`);
            log = `${log}${motivo}</br>`;
          }
        });
        const dataLog = {
          TITLE: I18n.translate('gop::LOG'),
          MESSAGE: log,
        };
        if (units.length === countUnit) {
          utilities.errorMessage('gop::ERROR_WARNING_ALL_UNITS');
        } else {
          utilities.warningMessage('gop::SUCCESS_WARNING');
        }
        utilities.createDefaultModal(dataLog);
        utilities.openDefaultModal();
      } else {
        utilities.successMessage();
      }
      utilities.hideDialogue();
    }).catch((e) => {
      utilities.errorMessage();
      console.warn(e);
    }).finally(() => {
      LoadingService.hide();
    });
  }
  
  /**
  * Creates a string with the budget types chosen by the user, indicating each visible budget type as Y or N
  *
  * @param {array} nrItemOperacional - list of errors coming from the budget preparation log.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public typeBudgetString(nrItemOperacional) {
    const matriz = ['N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N', 'N'];
    let stringTpOrca;
    nrItemOperacional.forEach((row) => {
      matriz.splice(row, 1, 'S');
      stringTpOrca = matriz.toString();
      stringTpOrca = stringTpOrca.replace(/,/g, '');
    });
    return stringTpOrca;
  }
  
  /**
  * Maps fields that are dynamic and dependent on each other to determine whether completion is required.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public dynamicFields() {
    const fields = {
      CDSUPERVISOR: {
        componentField: Metadata.getInstance<Select>('CDSUPERVISOR'),
        dependentFields: ['CDREGIAO', 'CDGRUPFILI', 'CDFILIAL'],
      },
      CDREGIAO: {
        componentField: Metadata.getInstance<Select>('CDREGIAO'),
        dependentFields: ['CDSUPERVISOR', 'CDGRUPFILI', 'CDFILIAL'],
      },
      CDGRUPFILI: {
        componentField: Metadata.getInstance<SelectMultiple>('CDGRUPFILI'),
        dependentFields: ['CDSUPERVISOR', 'CDREGIAO', 'CDFILIAL'],
      },
      CDFILIAL: {
        componentField: Metadata.getInstance<SelectMultiple>('CDFILIAL'),
        dependentFields: ['CDSUPERVISOR', 'CDGRUPFILI', 'CDREGIAO'],
      },
      CDSEGMER: {
        componentField: Metadata.getInstance<SelectMultiple>('CDSEGMER'),
        dependentFields: ['CDSUPERVISOR', 'CDGRUPFILI', 'CDREGIAO', 'CDFILIAL'],
      },
      CDOPERADOR: {
        componentField: Metadata.getInstance<SelectMultiple>('CDOPERADOR'),
        dependentFields: ['CDSUPERVISOR', 'CDGRUPFILI', 'CDREGIAO', 'CDFILIAL'],
      },
    };
    return fields;
  }
  
  /**
  * Add or clear the validation of a field by making it required or not.
  *
  * @param {array} fieldNames - dependent fields.
  * @param {string} operation - add or clear validation.
  * @param {array} fields - components fields.
  * @param {string} validationToAdd - indicates addition of required in the field.
  *
  * @access public
  * @author Grazielle Antunes
  */
  changeValidation(fieldNames: string[], operation: 'ADD' | 'CLEAR', fields, validationToAdd: string = 'required') {
    fieldNames.forEach((fieldName) => {
      const field = this.getFieldByName(fieldName, fields);
      if (field) {
        if (operation === 'CLEAR') field.clearValidations();
        else if (operation === 'ADD') field.addValidation(validationToAdd);
      }
    });
  }
  
  /**
  * Searches for the corresponding field component by name.
  *
  * @param {string} fieldName - field name.
  * @param {array} fields - array with the component of each field.
  *
  * @access public
  * @author Grazielle Antunes
  */
  getFieldByName(fieldName: string, fields) {
    const field = fieldName in fields ? fields[fieldName] : null;
    return field.componentField;
  }
  
  /**
  * Checks if the field is empty or null.
  *
  * @param {array} fieldNames - array with dependent fields.
  * @param {array} fields - array with the component of each field.
  *
  * @access public
  * @author Grazielle Antunes
  */
  isEmptyAllFields(fieldNames: string[], fields) {
    const utilities = new Utilities();
    return fieldNames.every((fieldName) => {
      const field = this.getFieldByName(fieldName, fields);
      if (field) {
        const value = field.value ? field.value : null;
        return utilities.isEmptyOrBlank(value);
      }
      throw new Error(I18n.translate('gop::INVALIDFIELD'));
    });
  }

  /**
  * Checks whether the field is empty or null and determines whether to clear or add required in dependent fields.
  *
  * @param {object} component - component that triggers the function.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public changeField({ component }) {
    const utilities = new Utilities();
    const fields = this.dynamicFields();
    const isEmptyField = utilities.isEmptyOrBlank(component.value);
    const fieldName = component.name;
    const dependentFields = fields[fieldName].dependentFields;
    if (isEmptyField) {
      if (this.isEmptyAllFields(dependentFields, fields)) {
        this.changeValidation([...dependentFields, fieldName], 'ADD', fields);
      }
    } else {
      this.changeValidation(dependentFields, 'CLEAR', fields);
    }
    if (component.name !== 'CDFILIAL') {
      this.clearFilial(fields);
    }
  }
  
  /**
  * Clears the value of the unit field if there is a change in fields that are used to filter the unit.
  *
  * @param {array} fields - array with the component of each field.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public clearFilial(fields) {
    const cdFilial = fields.CDFILIAL.componentField;
    if (cdFilial.value && cdFilial.value.length !== 0) {
      cdFilial.value = [];
    }
  }
  
  /**
  * Creates the filter for the unit field and updates it.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public createDataSourceFilter() {
    const fields = this.dynamicFields();
    const cdFilial = fields.CDFILIAL.componentField;
    const cdGrupFili = fields.CDGRUPFILI.componentField;
    const cdSegMer = fields.CDSEGMER.componentField;
    const cdOperador = fields.CDOPERADOR.componentField;
    const cdRegiao = fields.CDREGIAO.componentField;
    const cdSupervisor = fields.CDSUPERVISOR.componentField;
    cdFilial.datasource.setFilter({
      CDGRUPFILI: cdGrupFili.value && cdGrupFili.value.length ? cdGrupFili.value.join('_') : 'T',
      CDSEGMER: cdSegMer.value && cdSegMer.value.length ? cdSegMer.value.join('_') : 'T',
      CDOPERADOR: cdOperador.value && cdOperador.value.length ? cdOperador.value.join('_') : 'T',
      CDREGIAO: cdRegiao.value ? cdRegiao.value : 'T',
      CDSUPERVISOR: cdSupervisor.value ? cdSupervisor.value : 'T',
    });
  }
}
